//
//  AMShapedButton.m
//  Mandy
//
//  Created by Andreas on Mon Jul 28 2003.
//  Copyright (c) 2003 Andreas Mayer. All rights reserved.
//
//  2003-08-06 AM	updated to allow On/Off type buttons
//  2004-06-14 AM	fixed controlTintChanged: to not overwrite the alternate
//					image if there are no separate aqua/graphite versions


#import "AMShapedButton.h"
#import "AMDarkenedImage.h"


@interface AMShapedButton (Private)
- (void)updateImageBaseName;
- (void)controlTintChanged:(NSNotification *)aNotification;
- (BOOL)graphiteTint;
@end

@implementation AMShapedButton

- (id)init
{
	if (self = [super init]) {
		[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(controlTintChanged:) name:NSControlTintDidChangeNotification object:nil];
	}
	return self;
}

- (void)awakeFromNib
{
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(controlTintChanged:) name:NSControlTintDidChangeNotification object:nil];
	[self updateImageBaseName];
	[self controlTintChanged:nil];
}

- (void)setAlternateImage:(NSImage *)image
{
	[super setAlternateImage:image];
	[self updateImageBaseName];
}

- (void)updateImageBaseName
{
	// if name of image ends in "Aqua" switch to "Graphite" graphics if applicable
	NSString *imageName = [[self alternateImage] name];
	if (imageName) {
		NSRange aquaRange = [[[self alternateImage] name] rangeOfString:@"Aqua"];
		if (aquaRange.length > 0) {
			[self setImageBaseName:[imageName substringToIndex:aquaRange.location]];
		}
	}
}

- (NSString *)imageBaseName
{
    return imageBaseName;
}

- (void)setImageBaseName:(NSString *)newImageBaseName
{
    id old = nil;

    if (newImageBaseName != imageBaseName) {
        old = imageBaseName;
        imageBaseName = [newImageBaseName copy];
        [old release];
    }
}

/*
 //  Modified by Rams on Tues Feb 17 2004.
 //  Added additional mouseDown method:
 //  New method sets highlighted to YES only if mouse is over
 //  the image according to the hitTest method.

 // [Andreas: does not work for all types of buttons (e.g. those that fire continuously)]
 
 - (void)mouseDown:(NSEvent *)theEvent
{
	BOOL keepOn = YES;
	BOOL isInside = YES;
	NSPoint mouseLoc;
	
	[self highlight:YES];
	while (keepOn) {
		theEvent = [[self window] nextEventMatchingMask: NSLeftMouseUpMask | NSLeftMouseDraggedMask];
		mouseLoc = [theEvent locationInWindow];
		isInside = ([[self hitTest:mouseLoc] isEqual:self]) ? YES : NO;
		
		switch ([theEvent type]) {
			case NSLeftMouseDragged:
				[[self cell] setHighlighted:isInside];
				break;
			case NSLeftMouseUp:
				if (isInside) [self performClick:self];
				[self highlight:NO];
				keepOn = NO;
				break;
			default:
				// Ignore any other kind of event.
				break;
		}
		
	};
	
	return;
}
*/

- (void)drawRect:(NSRect)rect {
    // since standard buttons always draw a background, we have to do this ourselves
	// <sigh>
	NSImage *image = nil;
	float fraction = 1.0;
	if (![self isEnabled]) {
		// lighten image
		image = [self image];
		fraction = 0.4;
	} else {
		// (BUG: highlighted and on states should be visually distinctive)
		BOOL isHighlighted = [[self cell] isHighlighted];
		BOOL honorState = [[self cell] showsStateBy];
		BOOL isOn = (honorState && ([self state] == NSOnState));
		if (isHighlighted || isOn) {
			image = [self alternateImage];
			if (!image) {
				// should not happen since we explicitly
				// set that image in controlTintChanged
				image = [self image];
			}
			if (isHighlighted && honorState) {
				// darken image
				image = [image darkenedImageWithColor:[[NSColor controlHighlightColor] colorWithAlphaComponent:0.5]];
			}
		} else {
			// draw normal image
			image = [self image];
		}
	}
	NSPoint origin;
	if ([self isFlipped]) {
		NSSize size = [image size];
		origin = NSMakePoint(0, size.height);
	} else {
		origin = NSMakePoint(0,0);
	}
	[image dissolveToPoint:origin fraction:fraction];
}

- (BOOL)isOpaque
{
	return NO;
}

- (NSView *)hitTest:(NSPoint)aPoint
{
	// NSView instance method
	// Returns the farthest descendant of the receiver in the view hierarchy
	// (including itself) that contains aPoint, or nil if aPoint lies completely
	// outside the receiver. aPoint is in the coordinate system of the receiver's
	// superview, not of the receiver itself.

	NSView *result = self;
	NSPoint thePoint = [[self superview] convertPoint:aPoint toView:self];
	if (!NSPointInRect(thePoint, [self bounds])) {
		result = nil;
	} else {
		// get alpha mask of button image
		NSImageRep *imageRep = (NSImageRep *)[[[self image] representations] objectAtIndex:0];
		if (imageRep) {
			if ([imageRep hasAlpha]) {
				if ([imageRep isKindOfClass:[NSBitmapImageRep class]]) {
					if ([(NSBitmapImageRep *)imageRep isPlanar]) {
						NSLog(@"planar bitmap formats not supported");
					} else {
						if (([(NSBitmapImageRep *)imageRep bitsPerPixel]/[(NSBitmapImageRep *)imageRep samplesPerPixel]) != 8) {
							NSLog(@"Bits per Pixel/Samples per Pixel != 8: %ld/%ld", (long)[(NSBitmapImageRep *)imageRep bitsPerPixel], (long)[(NSBitmapImageRep *)imageRep samplesPerPixel]);
						} else {
							unsigned char *bitmapData = [(NSBitmapImageRep *)imageRep bitmapData];
							// calculate byte position
							// 1. skip rows
							int bytePos = trunc(thePoint.y) * [(NSBitmapImageRep *)imageRep bytesPerRow];
							// 2. add column
							bytePos += trunc(thePoint.x) * [(NSBitmapImageRep *)imageRep bitsPerPixel]/8;
							// 3. skip RGB values
							bytePos += 3;
							int alphaValue = bitmapData[bytePos];
							if (alphaValue < 128) {
								result = nil;
							}
						}
					}
				}
			}
		}
	}
	return result;
}

- (void)controlTintChanged:(NSNotification *)aNotification
{
	NSString *imageName = nil;
	NSImage *newImage = nil;
	if (imageBaseName) {
		if ([self graphiteTint]) {
			imageName = [imageBaseName stringByAppendingString:@"Graphite"];
		} else {
			imageName = [imageBaseName stringByAppendingString:@"Aqua"];
		}
		newImage = [NSImage imageNamed:imageName];
		if (!newImage) {
			newImage = [[self image] darkenedImageWithColor:[[NSColor colorForControlTint:NSDefaultControlTint] colorWithAlphaComponent:0.5]];
		}
		[super setAlternateImage:newImage];
	}
}

- (BOOL)graphiteTint
{
	//NSLog([[NSColor colorForControlTint:NSDefaultControlTint] colorNameComponent]);
	return ([[[NSColor colorForControlTint:NSDefaultControlTint] colorNameComponent] isEqualToString:@"graphiteControlTintColor"]);
}

@end
